<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 09, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
class Model_Cache_Backup extends Model_Cache_Abstract
{
    const EXPIRATION = Model_Cache::CACHE_EXPIRATION;
    const DEFAULT_EXPIRATION_DAYS = 31;

    /**
     * @param string $backupId
     * @param array  $backup
     */
    public function add($backupId, array $backup)
    {
        $this->_validateApiDetails($backup);

        $backups = $this->getAll();
        $backups[$backupId] = $backup;

        $this->_set($backups);
    }

    /**
     * validate api details.
     *
     * @param array $backup
     */
    private function _validateApiDetails(array $backup)
    {
        $apiDetails = $backup[self::API_DETAILS];

        $this->_validateIsArray($apiDetails);
        $this->_validateNotEmpty($apiDetails);
        $this->_validateKeysApiDetails($apiDetails);
    }

    /**
     * Validate api details contain required keys.
     *
     * @param array $apiDetails
     */
    private function _validateKeysApiDetails(array $apiDetails)
    {
        $requiredKeys = Model_Backups_Backup::getRequiredKeysApiDetails();
        foreach ($requiredKeys as $key) {
            if (!array_key_exists($key, $apiDetails)) {
                throw new Exception('Acronis Cache: Cannot cache api details - data missing: ' . $key);
            }
        }
    }

    /**
     * @return string
     */
    protected function _getCacheKey()
    {
        return Model_Cache::CACHE_BACKUP;
    }

    /**
     * @return string
     */
    protected function _getCacheName()
    {
        return 'backup';
    }

    /**
     * @param string $backupId
     * @param array  $users
     */
    public function addSwUsers($backupId, array $users)
    {
        $backup = $this->get($backupId);
        $backup[self::SW_USERS] = $users;
        $this->updateItem($backupId, $backup);
    }

    /**
     * Get sw users.
     *
     * @param string $backupId
     * @return array
     */
    public function getSwUsers($backupId)
    {
        $backup = $this->get($backupId);
        return $backup[self::SW_USERS];
    }


    /**
     * @param string $backupId
     * @return boolean
     */
    public function prescriptCached($backupId)
    {
        return !is_null($this->getSwUsers($backupId));
    }


    /**
     * @param string $backupId
     * @param array  $prescriptData
     */
    public function addPrescriptData($backupId, array $prescriptData)
    {
        $backup = $this->get($backupId);

        foreach ($prescriptData as $key => $value) {
            // TODO add validation
            $backup[$key] = $value;
        }

        $this->updateItem($backupId, $backup);
    }

    /**
     * add databases
     *
     * @param string $backupId
     * @param array  $dbs
     */
    public function addMappedDatabases($backupId, array $dbs)
    {
        $this->_validateDbs($dbs);

        $backup = $this->get($backupId);
        $backup[self::MAPPED_DATABASES] = $dbs;
        $this->updateItem($backupId, $backup);
    }

    /**
     * @param string $backupId
     * @return array
     */
    public function getMappedDatabases($backupId)
    {
        $backup = $this->get($backupId);
        return $backup[self::MAPPED_DATABASES];
    }

    /**
     * @param string $backupId
     * @param string $unixuser
     */
    public function getUserDatabases($backupId, $unixuser)
    {
        $mappedDatabases = $this->getMappedDatabases($backupId);
        $userDbs = array();

        foreach ($mappedDatabases as $db) {
            if ($db[self::UNIXUSER] === $unixuser) {
                $userDbs[] = $db;
            }
        }

        return $userDbs;
    }

    /**
     * add databases
     *
     * @param string $backupId
     * @param array  $dbs
     */
    public function addUnmappedDatabases($backupId, array $dbs)
    {
        $validateUnixuser = false;
        $this->_validateDbs($dbs, $validateUnixuser);

        $backup = $this->get($backupId);
        $backup[self::UNMAPPED_DATABASES] = $dbs;
        $this->updateItem($backupId, $backup);
    }


    /**
     * @param string $backupId
     * @return array
     */
    public function getUnmappedDatabases($backupId)
    {
        $backup = $this->get($backupId);
        return $backup[self::UNMAPPED_DATABASES];
    }

    /**
     * add dumps directory
     *
     * @param string $backupId
     * @param string $dir
     */
    public function addDumpsDirectoryUnmapped($backupId, $dir)
    {
        $backup = $this->get($backupId);
        $backup[self::DUMPS_DIR_UNMAPPED] = $dir;
        $this->updateItem($backupId, $backup);
    }

    /**
     * @param string $backupId
     * @return string
     */
    public function getUnmappedDumpsDirectory($backupId)
    {
        $backup = $this->get($backupId);
        return $backup[self::DUMPS_DIR_UNMAPPED];
    }

    /**
     * Get backups without prescript info mapped to them.
     *
     * @return array
     */
    public function getBackupsWithoutPrescriptInfo()
    {
        $missingPrescriptBackups = array_filter($this->getAll(), function ($backup) {
            return is_null($backup[self::SW_USERS]);
        });

        return array_values($missingPrescriptBackups);
    }

    /**
     * @param string $backupId
     * @return array
     */
    public function getApiData($backupId)
    {
        $backup = $this->get($backupId);
        return $backup[self::API_DETAILS];
    }

    /**
     * @return integer
     */
    private function _getExpirationTimestamp()
    {
        $expireAfterDays = IW::Cache()->get(self::EXPIRATION);
        if ($expireAfterDays === null) {
            $expireAfterDays = self::DEFAULT_EXPIRATION_DAYS;
        }

        return strtotime("-$expireAfterDays days");
    }

    /**
     * @param integer $days
     */
    public function setExpireAfterDays($days)
    {
        if (!is_numeric($days)) {
            throw new Exception_Acronis_InvalidCache('setExpireAfterDays: input days must be numeric');
        }

        $this->_setCache(self::EXPIRATION, $days);
    }

    /**
     * Delete expired backups. Set expiration in nw controller setBackupExpiration
     */
    public function deleteExpiredBackups()
    {
        $backups = [];

        foreach ($this->getAll() as $backupId => $backup) {
            $Backup = new Model_Backups_Backup($backup[self::API_DETAILS]);

            if (!$this->expired($Backup->getTimeStamp())) {
                $backups[$backupId] = $backup;
            }
        }

        $this->_set($backups);
    }

    /**
     * @param integer $timestamp
     */
    public function expired($timestamp)
    {
        $expireAfter  = $this->_getExpirationTimestamp();
        $expired = $timestamp < $expireAfter;

        return $expired;
    }
}
