import { displayErrorMessage, toggleVisibility } from '../../helpers.js';
import { xhr } from '../../xhr.js';
import { hideInProgress, showInProgress, showSuccess, updateProgress } from '../backupsHelpers.js';

let ignoreAndResumeId = 'ignore-and-resume';

let $errorMessageContainer = $('#ab-restore-error-message');
let $progressMessageDiv = $('#progress-error-message');
let $progressDetailsDiv = $('#progress-error-details');
let $linkDisplayErrorDetails = $('#display-progress-details');


const getProgress = (planId) => setTimeout(() => {
    sendGetProgressRequest(planId)
        .done(res => handleGetProgressResponse(res))
        .fail((res) => displayErrorMessageNoResponse(res));
}, 5000);

const displayErrorMessageNoResponse = (response) => {
    displayErrorMessage('Something went wrong! The server is not responsive');
}

const handleGetProgressResponse = (response) => {
    if (xhr.responseSuccessful(response)) {
        handleGetProgressSuccessfulResponse(response);
    } else {
        // should not get here. Failed action or form issue etc. would generate this error.
        displayErrorMessageIWorxResponseError(response)
    }
}

const displayErrorMessageIWorxResponseError = (response) => {
    console.log('iworx ctrl did not send a successful reply');
    console.log(xhr.getIworxResponseCode(response));
    console.log(xhr.getResponseData(response));
    console.log(xhr.getResponseText(response));

    let data = xhr.getResponseData(response);

    displayErrorMessage(data.message);
}

const handleGetProgressSuccessfulResponse = (response) => {
    let data = xhr.getResponseData(response);
    updateProgress(data.progress);

    if (data.recoveryError) {
        handleProgressError(data);
    } else {
        handleProgressSuccess(response);
    }
}

const handleProgressSuccess = (response) => {
    let data = xhr.getResponseData(response);

    updateProgress(data.progress);

    if (data.progress !== '100%') {
        getProgress(data.planId);
    } else {
        hideInProgress();
        showSuccess('Successfully recovered files!');
    }
}

const handleProgressError = (data) => {
    hideInProgress();
    let error = data.recoveryError;

    $progressMessageDiv.html(error.message);
    $progressDetailsDiv.html(error.details);

    $linkDisplayErrorDetails.off('click');
    $linkDisplayErrorDetails.on('click', () => toggleVisibility($progressDetailsDiv));

    if ($errorMessageContainer.is(':hidden')) {
        toggleVisibility($errorMessageContainer);
    }


    if (data.state === 'paused') {
        appendIgnoreErrorActionLink();
        registerEventListenerResumeRecovery(data);
    } else {
        removeIgnoreLink();
    }
}

const removeIgnoreLink = () =>
{
    if (ignoreLinkExists()) {
        $(`#${ignoreAndResumeId}`).remove();
    }
}

const appendIgnoreErrorActionLink = () => {

    if (!ignoreLinkExists()) {
        let link = `<div id="${ignoreAndResumeId}"><a>Ignore error and resume recovery</a></div>`;
        $errorMessageContainer.append(link);
    }
}

const ignoreLinkExists = () => $(`#${ignoreAndResumeId}`).length > 0;

const registerEventListenerResumeRecovery = (previousData) => {
    $(`#${ignoreAndResumeId}`).off('click');
    $(`#${ignoreAndResumeId}`).on('click', () => resumeRecovery(previousData));
}

const resumeRecovery = (previousData) => {
    toggleVisibility($errorMessageContainer);
    showInProgress('resuming recovery...', previousData.progress);

    let action = 'resumeRecovery';
    let args = { ab_recovery_id: previousData.recoveryId };

    xhr.postRequest(action, args)
        .done(res => handleResumeRecoveryResponse(res, previousData))
        .fail(res => displayErrorMessageNoResponse(res));
}

const handleResumeRecoveryResponse = (res, previousData) => {
    if (xhr.responseSuccessful(res)) {
        sendGetProgressRequest(previousData.planId)
            .done(res => checkProgressAfterResume(res, previousData))
    } else {
        displayErrorMessageIWorxResponseError(res);
    }
}

const checkProgressAfterResume = (res, previousData) => {
    let data = xhr.getResponseData(res);
    let state = data.state;
    let planId = previousData.planId;

    if (state === 'paused') {
        setTimeout(() => {
            sendGetProgressRequest(planId)
            .done(res => handleResumeRecoveryResponse(res, previousData))
            .fail(res => displayErrorMessageNoResponse(res));
        }, 3000);
    } else {
        handleProgressSuccess(res);
    }
}

const sendGetProgressRequest = (planId) => {
    let action = 'getRecoveryProgress';
    let args = { ab_plan_id: planId };

    return xhr.postRequest(action, args);
}

export const recoveryProgress = {
    getProgress
}